@@ -208,7 +208,7 @@ static void init_getbits(struct LhADecrData *dat)
 
 /* ------------------------------------------------------------------------ */
 
-static void make_table(struct LhADecrData *dat, int16 nchar, uint8 bitlen[], int16 tablebits, uint16 table[])
+static int make_table(struct LhADecrData *dat, int16 nchar, uint8 bitlen[], int16 tablebits, uint16 table[], int table_size)
 {
   uint16 count[17];  /* count of bitlen */
   uint16 weight[17]; /* 0x10000ul >> bitlen */
@@ -243,7 +243,7 @@ static void make_table(struct LhADecrData *dat, int16 nchar, uint8 bitlen[], int
   if(total & 0xFFFF)
   {
     dat->error = 1;
-    return;
+    return -1;
   }
 
   /* shift data for make table. */
@@ -256,9 +256,16 @@ static void make_table(struct LhADecrData *dat, int16 nchar, uint8 bitlen[], int
   /* initialize */
   j = start[tablebits + 1] >> m;
   k = 1 << tablebits;
-  if(j != 0)
+  if(j != 0) {
+
+    /* Sanity check */
+    if (k > table_size) {
+      return -1;
+    }
+
     for(i = j; i < k; i++)
       table[i] = 0;
+  }
 
   /* create table and tree */
   for(j = 0; j < nchar; j++)
@@ -267,8 +274,14 @@ static void make_table(struct LhADecrData *dat, int16 nchar, uint8 bitlen[], int
     if(k == 0)
       continue;
     l = start[k] + weight[k];
+
     if(k <= tablebits)
     {
+      /* Sanity check */
+      if (l > table_size) {
+        return -1;
+      }
+
       /* code in table */
       for(i = start[k]; i < l; i++)
         table[i] = j;
@@ -298,12 +311,12 @@ static void make_table(struct LhADecrData *dat, int16 nchar, uint8 bitlen[], int
     start[k] = l;
   }
   
-  return;
+  return 0;
 }
 
 /* ------------------------------------------------------------------------ */
 
-static void read_pt_len(struct LhADecrData *dat, int16 nn, int16 nbit, int16 i_special)
+static int read_pt_len(struct LhADecrData *dat, int16 nn, int16 nbit, int16 i_special)
 {
   int16 i, c, n;
 
@@ -343,11 +356,14 @@ static void read_pt_len(struct LhADecrData *dat, int16 nn, int16 nbit, int16 i_s
     }
     while(i < nn)
       dat->d.st.pt_len[i++] = 0;
-    make_table(dat, nn, dat->d.st.pt_len, 8, dat->d.st.pt_table);
+    if (make_table(dat, nn, dat->d.st.pt_len, 8, dat->d.st.pt_table, 256) < 0)
+      return -1;
   }
+
+  return 0;
 }
 
-static void read_c_len(struct LhADecrData *dat)
+static int read_c_len(struct LhADecrData *dat)
 {
   int16 i, c, n;
 
@@ -396,20 +412,26 @@ static void read_c_len(struct LhADecrData *dat)
     }
     while(i < NC)
       dat->d.st.c_len[i++] = 0;
-    make_table(dat, NC, dat->d.st.c_len, 12, dat->d.st.c_table);
+    if (make_table(dat, NC, dat->d.st.c_len, 12, dat->d.st.c_table, 4096) < 0)
+      return -1;
   }
+
+  return 0;
 }
 
-static uint16 decode_c_st1(struct LhADecrData *dat)
+static int decode_c_st1(struct LhADecrData *dat)
 {
   uint16 j, mask;
 
   if(!dat->d.st.blocksize)
   {
     dat->d.st.blocksize = getbits(dat, 16);
-    read_pt_len(dat, NT, TBIT, 3);
-    read_c_len(dat);
-    read_pt_len(dat, dat->d.st.np, dat->d.st.pbit, -1);
+    if (read_pt_len(dat, NT, TBIT, 3) < 0)
+      return -1;
+    if (read_c_len(dat) < 0)
+      return -1;
+    if (read_pt_len(dat, dat->d.st.np, dat->d.st.pbit, -1) < 0)
+      return -1;
   }
   dat->d.st.blocksize--;
   j = dat->d.st.c_table[dat->bitbuf >> 4];
@@ -458,7 +480,7 @@ static uint16 decode_p_st1(struct LhADecrData *dat)
   return j;
 }
 
-static void decode_start_st1(struct LhADecrData *dat)
+static int decode_start_st1(struct LhADecrData *dat)
 {
   if(dat->DicBit <= 13)
   {
@@ -475,6 +497,8 @@ static void decode_start_st1(struct LhADecrData *dat)
   }
   init_getbits(dat);
 //  dat->d.st.blocksize = 0; /* done automatically */
+
+  return 0;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -714,7 +738,7 @@ static void update_c(struct LhADecrData *dat, int32 p)
   } while(q != ROOT_C);
 }
 
-static uint16 decode_c_dyn(struct LhADecrData *dat)
+static int decode_c_dyn(struct LhADecrData *dat)
 {
   int32 c;
   int16 buf, cnt;
@@ -808,7 +832,7 @@ static void ready_made(struct LhADecrData *dat, int32 method)
   }
 }
 
-static void decode_start_fix(struct LhADecrData *dat)
+static int decode_start_fix(struct LhADecrData *dat)
 {
   dat->d.st.n_max = 314;
   dat->d.st.maxmatch = 60;
@@ -816,7 +840,10 @@ static void decode_start_fix(struct LhADecrData *dat)
   dat->d.st.np = 1 << (12 - 6);
   start_c_dyn(dat);
   ready_made(dat, 0);
-  make_table(dat, dat->d.st.np, dat->d.st.pt_len, 8, dat->d.st.pt_table);
+  if (make_table(dat, dat->d.st.np, dat->d.st.pt_len, 8, dat->d.st.pt_table, 256) < 0)
+    return -1;
+
+  return 0;
 }
 
 static uint16 decode_p_st0(struct LhADecrData *dat)
@@ -855,7 +882,7 @@ static void decode_start_st0(struct LhADecrData *dat)
   dat->d.st.np = 1 << (MAX_DICBIT - 6);
 }
 
-static void read_tree_c(struct LhADecrData *dat) /* read tree from file */
+static int read_tree_c(struct LhADecrData *dat) /* read tree from file */
 {
   int32 i, c;
 
@@ -872,10 +899,13 @@ static void read_tree_c(struct LhADecrData *dat) /* read tree from file */
       memset(dat->d.st.c_len, 0, N1);
       for(i = 0; i < 4096; i++)
         dat->d.st.c_table[i] = c;
-      return;
+      return 0;
     }
   }
-  make_table(dat, N1, dat->d.st.c_len, 12, dat->d.st.c_table);
+  if (make_table(dat, N1, dat->d.st.c_len, 12, dat->d.st.c_table, 4096) < 0)
+    return -1;
+
+  return 0;
 }
 
 static void read_tree_p(struct LhADecrData *dat) /* read tree from file */
@@ -898,14 +928,15 @@ static void read_tree_p(struct LhADecrData *dat) /* read tree from file */
   }
 }
 
-static uint16 decode_c_st0(struct LhADecrData *dat)
+static int decode_c_st0(struct LhADecrData *dat)
 {
   int32 i, j;
 
   if(!dat->d.st.blocksize) /* read block head */
   {
     dat->d.st.blocksize = getbits(dat, BUFBITS); /* read block blocksize */
-    read_tree_c(dat);
+    if (read_tree_c(dat) < 0)
+      return -1;
     if(getbits(dat, 1))
     {
       read_tree_p(dat);
@@ -914,7 +945,8 @@ static uint16 decode_c_st0(struct LhADecrData *dat)
     {
       ready_made(dat, 1);
     }
-    make_table(dat, NP, dat->d.st.pt_len, 8, dat->d.st.pt_table);
+    if (make_table(dat, NP, dat->d.st.pt_len, 8, dat->d.st.pt_table, 256) < 0)
+      return -1;
   }
   dat->d.st.blocksize--;
   j = dat->d.st.c_table[dat->bitbuf >> 4];
@@ -1344,8 +1376,8 @@ static int32 LhA_Decrunch(FILE *in, FILE *out, int size, uint32 Method)
   int32 err = 0;
 
   if((dd = calloc(sizeof(struct LhADecrData), 1))) {
-    void (*DecodeStart)(struct LhADecrData *);
-    uint16 (*DecodeC)(struct LhADecrData *);
+    int (*DecodeStart)(struct LhADecrData *);
+    int (*DecodeC)(struct LhADecrData *);
     uint16 (*DecodeP)(struct LhADecrData *);
 
     /* most often used stuff */
@@ -1438,7 +1470,12 @@ static int32 LhA_Decrunch(FILE *in, FILE *out, int size, uint32 Method)
 */
           memset(text, ' ', (size_t) dicsiz);
 
-        DecodeStart(dd);
+        if (DecodeStart(dd) < 0) {
+          free(dd->text);
+          free(dd);
+          return -1;
+        }
+
         --dicsiz; /* now used with AND */
         while(1)
         {
@@ -1446,6 +1483,11 @@ static int32 LhA_Decrunch(FILE *in, FILE *out, int size, uint32 Method)
             break;
 
           c = DecodeC(dd);
+          if (c < 0) {
+            free(dd->text);
+            free(dd);
+            return -1;  
+          }
 
 	  if (dd->error)
 	    break;
@@ -1688,7 +1730,8 @@ static int get_header(FILE *f, struct lha_data *data)
 }
 
 static int test_lha(unsigned char *b) {
-	return b[2] == '-' && b[3] == 'l' && b[4] == 'h';
+	return b[2] == '-' && b[3] == 'l' && b[4] == 'h' && b[6] == '-' &&
+		b[20] <= 3;
 }
 
 static int decrunch_lha(FILE *in, FILE *out)
