@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -242,7 +242,17 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		break;
 
 	case FX_SETPAN:		/* Set pan */
-		xc->pan.val = fxp;
+		/* From OpenMPT PanOff.xm:
+		 * "Another chapter of weird FT2 bugs: Note-Off + Note Delay
+		 *  + Volume Column Panning = Panning effect is ignored."
+		 */
+		if (!HAS_QUIRK(QUIRK_FT2BUGS)		/* If not FT2 */
+		    || fnum == 0			/* or not vol column */
+		    || e->note != XMP_KEY_OFF		/* or not keyoff */
+		    || e->fxt != FX_EXTENDED		/* or not delay */
+		    || MSN(e->fxp) != EX_DELAY) {
+			xc->pan.val = fxp;
+		}
 		break;
 	case FX_OFFSET:		/* Set sample offset */
 		EFFECT_MEMORY(fxp, xc->offset.memory);
@@ -353,20 +363,18 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			break;
 		case EX_VIBRATO_WF:	/* Set vibrato waveform */
 			fxp &= 3;
-			if (HAS_QUIRK(QUIRK_S3MLFO) && fxp == 2)
-				fxp |= 0x10;
 			set_lfo_waveform(&xc->vibrato.lfo, fxp);
 			break;
 		case EX_FINETUNE:	/* Set finetune */
 			fxp <<= 4;
-			if (!HAS_QUIRK(QUIRK_XMFINE) || note > 0)
+			if (!HAS_QUIRK(QUIRK_FT2BUGS) || note > 0)
 				goto fx_finetune;
 			break;
 		case EX_PATTERN_LOOP:	/* Loop pattern */
 			if (fxp == 0) {
 				/* mark start of loop */
 				f->loop[chn].start = p->row;
-				if (HAS_QUIRK(QUIRK_FT2LOOP))
+				if (HAS_QUIRK(QUIRK_FT2BUGS))
 				  p->flow.jumpline = p->row;
 			} else {
 				/* end of loop */
@@ -380,10 +388,8 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 							    p->row + 1;
 					}
 				} else {
-					if (f->loop[chn].start <= p->row) {
-						f->loop[chn].count = fxp;
-						f->loop_chn = ++chn;
-					}
+					f->loop[chn].count = fxp;
+					f->loop_chn = ++chn;
 				}
 			}
 			break;
@@ -475,14 +481,15 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		}
 		break;
 	case FX_S3M_BPM:	/* Set S3M BPM */
-            fx_s3m_bpm:
-		if (fxp >= 0x20) {
-			if (fxp < XMP_MIN_BPM)
-				fxp = XMP_MIN_BPM;
-			p->bpm = fxp;
-			p->frame_time = m->time_factor * m->rrate / p->bpm;
-		}
+	    fx_s3m_bpm: {
+		/* Lower time factor in MED allows lower BPM values */
+		int min_bpm = (int)(0.5 + m->time_factor * XMP_MIN_BPM / 10);
+		if (fxp < min_bpm)
+			fxp = min_bpm;
+		p->bpm = fxp;
+		p->frame_time = m->time_factor * m->rrate / p->bpm;
 		break;
+	}
 
 #ifndef LIBXMP_CORE_DISABLE_IT
 	case FX_IT_BPM:		/* Set IT BPM */
@@ -547,10 +554,24 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		xc->keyoff = fxp + 1;
 		break;
 	case FX_ENVPOS:		/* Set envelope position */
-		/* FIXME: Add OpenMPT quirk */
+		/* From OpenMPT SetEnvPos.xm:
+		 * "When using the Lxx effect, Fasttracker 2 only sets the
+		 *  panning envelope position if the volume envelope’s sustain
+		 *  flag is set.
+		 */
+		if (HAS_QUIRK(QUIRK_FT2BUGS)) {
+			struct xmp_instrument *instrument;
+			instrument = get_instrument(ctx, xc->ins);
+			if (instrument != NULL) {
+				if (instrument->aei.flg & XMP_ENVELOPE_SUS) {
+					xc->p_idx = fxp;
+				}
+			}
+		} else {
+			xc->p_idx = fxp;
+		}
 		xc->v_idx = fxp;
 		xc->f_idx = fxp;
-		xc->p_idx = fxp;
 		break;
 	case FX_PANSLIDE:	/* Pan slide (XM) */
 		EFFECT_MEMORY(fxp, xc->pan.memory);
@@ -685,8 +706,22 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			virt_setnna(ctx, chn, XMP_INST_NNA_FADE);
 			break;
 		case 7:	/* Turn off volume envelope */
+			SET_PER(VENV_PAUSE);
 			break;
 		case 8:	/* Turn on volume envelope */
+			RESET_PER(VENV_PAUSE);
+			break;
+		case 9:	/* Turn off pan envelope */
+			SET_PER(PENV_PAUSE);
+			break;
+		case 0xa:	/* Turn on pan envelope */
+			RESET_PER(PENV_PAUSE);
+			break;
+		case 0xb:	/* Turn off pitch envelope */
+			SET_PER(FENV_PAUSE);
+			break;
+		case 0xc:	/* Turn on pitch envelope */
+			RESET_PER(FENV_PAUSE);
 			break;
 		}
 		break;
@@ -715,7 +750,13 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	 * nibble is set (i.e. F30 is the same as F03).
 	 */
 	case FX_ICE_SPEED:
-		p->st26_speed = (MSN(fxp) << 8) | LSN(fxp);
+		if (fxp) {
+			if (LSN(fxp)) {
+				p->st26_speed = (MSN(fxp) << 8) | LSN(fxp);
+			} else {
+				p->st26_speed = MSN(fxp);
+			}
+		}
 		break;
 
 	case FX_VOLSLIDE_UP:	/* Vol slide with uint8 arg */
