@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -139,6 +139,8 @@ static int decrunch(HIO_HANDLE **h, char *filename, char **temp)
 	for (i = 0; depacker_list[i] != NULL; i++) {
 		if (depacker_list[i]->test(b)) {
 			depacker = depacker_list[i];
+			D_(D_INFO "Use depacker %d", i);
+			break;
 		}
 	}
 
@@ -159,13 +161,16 @@ static int decrunch(HIO_HANDLE **h, char *filename, char **temp)
 
 	fseek(f, 0, SEEK_SET);
 
-	if (depacker == NULL && cmd == NULL)
+	if (depacker == NULL && cmd == NULL) {
+		D_(D_INFO "Not packed");
 		return 0;
+	}
 
 #if defined ANDROID || defined __native_client__
 	/* Don't use external helpers in android */
-	if (cmd)
+	if (cmd) {
 		return 0;
+	}
 #endif
 
 	D_(D_WARN "Depacking file... ");
@@ -175,12 +180,14 @@ static int decrunch(HIO_HANDLE **h, char *filename, char **temp)
 
 	/* Depack file */
 	if (cmd) {
+		D_(D_INFO "External depacker: %s", cmd);
 		if (execute_command(cmd, filename, t) < 0) {
 			D_(D_CRIT "failed");
 			fclose(t);
 			return -1;
 		}
 	} else if (depacker) {
+		D_(D_INFO "Internal depacker");
 		if (depacker->depack(f, t) < 0) {
 			D_(D_CRIT "failed");
 			fclose(t);
@@ -334,6 +341,7 @@ static int load_module(xmp_context opaque, HIO_HANDLE *h)
 {
 	struct context_data *ctx = (struct context_data *)opaque;
 	struct module_data *m = &ctx->m;
+	struct xmp_module *mod = &m->mod;
 	int i, ret;
 	int test_result, load_result;
 
@@ -368,7 +376,13 @@ static int load_module(xmp_context opaque, HIO_HANDLE *h)
 		return -XMP_ERROR_LOAD;
 	}
 
-	adjust_string(m->mod.name);
+	/* Sanity check */
+	if (mod->chn > XMP_MAX_CHANNELS || mod->len > XMP_MAX_MOD_LENGTH) {
+		xmp_release_module(opaque);
+		return -XMP_ERROR_LOAD;
+	}
+
+	adjust_string(mod->name);
 	load_epilogue(ctx);
 
 	ret = prepare_scan(ctx);
@@ -534,24 +548,26 @@ void xmp_release_module(xmp_context opaque)
 	release_module_extras(ctx);
 #endif
 
-	for (i = 0; i < mod->trk; i++)
-		free(mod->xxt[i]);
-	if (mod->trk > 0)
+	if (mod->xxt != NULL) {
+		for (i = 0; i < mod->trk; i++) {
+			free(mod->xxt[i]);
+		}
 		free(mod->xxt);
+	}
 
-	for (i = 0; i < mod->pat; i++)
-		free(mod->xxp[i]);
-	if (mod->pat > 0)
+	if (mod->xxp != NULL) {
+		for (i = 0; i < mod->pat; i++) {
+			free(mod->xxp[i]);
+		}
 		free(mod->xxp);
+	}
 
 	if (mod->xxi != NULL) {
 		for (i = 0; i < mod->ins; i++) {
 			free(mod->xxi[i].sub);
 			free(mod->xxi[i].extra);
 		}
-		if (mod->ins > 0) {
-			free(mod->xxi);
-		}
+		free(mod->xxi);
 	}
 
 	if (mod->xxs != NULL) {
@@ -559,9 +575,7 @@ void xmp_release_module(xmp_context opaque)
 			if (mod->xxs[i].data != NULL)
 				free(mod->xxs[i].data - 4);
 		}
-		if (mod->smp > 0) {
-			free(mod->xxs);
-		}
+		free(mod->xxs);
 	}
 
 	if (m->scan_cnt) {
