@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -153,8 +153,15 @@ char *adjust_string(char *s)
 
 static void check_envelope(struct xmp_envelope *env)
 {
-	if (env->npt <= 0 || env->lps >= env->npt || env->lpe >= env->npt)
+	/* Disable envelope if invalid number of points */
+	if (env->npt <= 0 || env->npt > XMP_MAX_ENV_POINTS) {
+		env->flg &= ~XMP_ENVELOPE_ON;
+	}
+
+	/* Disable envelope loop if invalid loop parameters */
+	if (env->lps >= env->npt || env->lpe >= env->npt) {
 		env->flg &= ~XMP_ENVELOPE_LOOP;
+	}
 }
 
 void load_prologue(struct context_data *ctx)
@@ -163,8 +170,7 @@ void load_prologue(struct context_data *ctx)
 	int i;
 
 	/* Reset variables */
-	memset(m->mod.name, 0, XMP_NAME_SIZE);
-	memset(m->mod.type, 0, XMP_NAME_SIZE);
+	memset(&m->mod, 0, sizeof (struct xmp_module));
 	m->rrate = PAL_RATE;
 	m->c4rate = C4_PAL_RATE;
 	m->volbase = 0x40;
@@ -209,6 +215,13 @@ void load_epilogue(struct context_data *ctx)
 
     	mod->gvl = m->gvolbase;
 
+	/* Sanity check for module parameters */
+	CLAMP(mod->len, 0, 255);
+	CLAMP(mod->pat, 0, 255);
+	CLAMP(mod->ins, 0, 255);
+	CLAMP(mod->smp, 0, 255);
+	CLAMP(mod->chn, 0, XMP_MAX_CHANNELS);
+
 	/* Fix cases where the restart value is invalid e.g. kc_fall8.xm
 	 * from http://aminet.net/mods/mvp/mvp_0002.lha (reported by
 	 * Ralf Hoffmann <ralf@boomerangsworld.de>)
@@ -217,13 +230,11 @@ void load_epilogue(struct context_data *ctx)
 		mod->rst = 0;
 	}
 
-	/* Sanity check */
-	if (mod->spd == 0) {
+	/* Sanity check for tempo and BPM */
+	if (mod->spd <= 0 || mod->spd > 255) {
 		mod->spd = 6;
 	}
-	if (mod->bpm == 0) {
-		mod->bpm = 125;
-	}
+	CLAMP(mod->bpm, XMP_MIN_BPM, 255);
 
 	/* Set appropriate values for instrument volumes and subinstrument
 	 * global volumes when QUIRK_INSVOL is not set, to keep volume values
