@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -34,11 +34,13 @@
 
 int instrument_init(struct xmp_module *mod)
 {
-	mod->xxi = calloc(sizeof (struct xmp_instrument), mod->ins);
-	if (mod->xxi == NULL)
-		return -1;
+	if (mod->ins > 0) {
+		mod->xxi = calloc(sizeof (struct xmp_instrument), mod->ins);
+		if (mod->xxi == NULL)
+			return -1;
+	}
 
-	if (mod->smp) {
+	if (mod->smp > 0) {
 		mod->xxs = calloc (sizeof (struct xmp_sample), mod->smp);
 		if (mod->xxs == NULL)
 			return -1;
