@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -265,6 +265,14 @@ static void read_envelope(struct xmp_envelope *ei, struct it_envelope *env, HIO_
 
     env->flg = hio_read8(f);
     env->num = hio_read8(f);
+
+    /* Sanity check */
+    if (env->num >= XMP_MAX_ENV_POINTS) {
+	env->flg = 0;
+	env->num = 0;
+	return;
+    }
+
     env->lpb = hio_read8(f);
     env->lpe = hio_read8(f);
     env->slb = hio_read8(f);
@@ -359,6 +367,11 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
     mod->smp = ifh.smpnum;
     mod->pat = ifh.patnum;
 
+    /* Sanity check */
+    if (mod->len > 255 || mod->ins > 255 || mod->smp > 255 || mod->pat > 255) {
+	goto err;
+    }
+
     if (mod->ins) {
         pp_ins = calloc(4, mod->ins);
         if (pp_ins == NULL)
@@ -824,6 +837,12 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	    xxs->flg = XMP_SAMPLE_16BIT;
 	}
 	xxs->len = ish.length;
+
+	/* Sanity check */
+	if (xxs->len > MAX_SAMPLE_SIZE) {
+	    goto err4;
+	}
+
 	xxs->lps = ish.loopbeg;
 	xxs->lpe = ish.loopend;
 
@@ -913,12 +932,12 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 							&mod->xxs[i], buf);
 		if (ret < 0) {
 		    free(buf);
-		    return -1;
+                    goto err4;
 		}
 		free (buf);
 	    } else {
 		if (load_sample(m, f, cvt, &mod->xxs[i], NULL) < 0)
-		    return -1;
+                    goto err4;
 	    }
 	}
     }
