@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -211,6 +211,16 @@ static int rtm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	hio_read(&rh.panning, 32, 1, f);
 	rh.extraDataSize = hio_read32l(f);
 
+	/* Sanity check */
+	if (rh.nposition > 255)
+		return -1;
+	if (rh.ntrack > 32)
+		return -1;
+	if (rh.npattern > 255)
+		return -1;
+	if (rh.ninstr > 255)
+		return -1;
+
 	if (version >= 0x0112)
 		hio_seek(f, 32, SEEK_CUR);		/* skip original name */
 
