@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -85,6 +85,11 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 	xph.length = hio_read32l(f);
 	xph.packing = hio_read8(f);
 	xph.rows = version > 0x0102 ? hio_read16l(f) : hio_read8(f) + 1;
+
+	/* Sanity check */
+	if (xph.rows > 256)
+	    goto err;
+
 	xph.datasize = hio_read16l(f);
 	hio_seek(f, xph.length - headsize, SEEK_CUR);
 
@@ -93,31 +98,56 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 	    r = 0x100;
 
 	if (pattern_tracks_alloc(mod, i, r) < 0)
-	    return -1;
+	    goto err;
 
 	if (xph.datasize) {
-	    pat = patbuf = calloc(1, xph.datasize);
-	    if (patbuf == NULL)
-		return -1;
+	    int size = xph.datasize;
 
-	    hio_read(patbuf, 1, xph.datasize, f);
+	    pat = patbuf = calloc(1, size);
+	    if (patbuf == NULL)
+		goto err;
+
+	    hio_read(patbuf, 1, size, f);
 	    for (j = 0; j < (mod->chn * r); j++) {
-		if ((pat - patbuf) >= xph.datasize)
-		    break;
+
+		/*if ((pat - patbuf) >= xph.datasize)
+		    break;*/
+
 		event = &EVENT(i, j % mod->chn, j / mod->chn);
+
+		if (--size < 0)
+		    goto err2;
+
 		if ((b = *pat++) & XM_EVENT_PACKING) {
-		    if (b & XM_EVENT_NOTE_FOLLOWS)
+		    if (b & XM_EVENT_NOTE_FOLLOWS) {
+			if (--size < 0)
+			    goto err2;
 			event->note = *pat++;
-		    if (b & XM_EVENT_INSTRUMENT_FOLLOWS)
+		    }
+		    if (b & XM_EVENT_INSTRUMENT_FOLLOWS) {
+			if (--size < 0)
+			    goto err2;
 			event->ins = *pat++;
-		    if (b & XM_EVENT_VOLUME_FOLLOWS)
+		    }
+		    if (b & XM_EVENT_VOLUME_FOLLOWS) {
+			if (--size < 0)
+			    goto err2;
 			event->vol = *pat++;
+		    }
 		    if (b & XM_EVENT_FXTYPE_FOLLOWS) {
+			if (--size < 0)
+			    goto err2;
 			event->fxt = *pat++;
 		    }
-		    if (b & XM_EVENT_FXPARM_FOLLOWS)
+		    if (b & XM_EVENT_FXPARM_FOLLOWS) {
+			if (--size < 0)
+			    goto err2;
 			event->fxp = *pat++;
+		    }
 		} else {
+                    size -=4;
+		    if (size < 0)
+			goto err2;
 		    event->note = b;
 		    event->ins = *pat++;
 		    event->vol = *pat++;
@@ -125,14 +155,23 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 		    event->fxp = *pat++;
 		}
 
+		/* Sanity check */
+		switch (event->fxt) {
+		case 18: case 19:
+		case 22: case 23: case 24:
+		case 26:
+		case 28:
+		case 30: case 31: case 32:
+			event->fxt = 0;
+		}
+		if (event->fxt > 34) {
+			event->fxt = 0;
+		}
+
 		if (event->note == 0x61) {
 		    /* See OpenMPT keyoff+instr.xm test case */
-		    if (event->fxt == 0x0e && (event->fxp >> 4) == 0x0d) {
-			if (event->ins) {
-				event->note = event->fxp = event->fxt = 0;
-			} else {
-				event->note = XMP_KEY_OFF;
-			}
+		    if (event->fxt == 0x0e && MSN(event->fxp) == 0x0d) {
+			event->note = XMP_KEY_OFF;
 		    } else {
 		    	event->note = event->ins ? XMP_KEY_FADE : XMP_KEY_OFF;
 		    }
@@ -232,7 +271,7 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 		}
 		event->vol = 0;
 	    }
-	    free (patbuf);
+	    free(patbuf);
 	}
     }
 
@@ -241,18 +280,23 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 	int t = i * mod->chn;
 
 	if (pattern_alloc(mod, i) < 0)
-	    return -1;
+	    goto err;
 
 	mod->xxp[i]->rows = 64;
 
 	if (track_alloc(mod, t, 64) < 0)
-	    return -1;
+	    goto err;
 
 	for (j = 0; j < mod->chn; j++)
 	    mod->xxp[i]->index[j] = t;
     }
 
     return 0;
+
+  err2:
+    free(patbuf);
+  err:
+    return -1;
 }
 
 /* Packed structures size */
@@ -286,7 +330,6 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 	 * will not work if file has trailing garbage.
 	 */
 	if (hio_eof(f)) {
-		mod->ins = i;
 		break;
 	}
 
@@ -297,7 +340,6 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 
 	/* Sanity check */
 	if (xih.samples > 0x10 || (xih.samples > 0 && xih.sh_size > 0x100)) {
-		mod->ins = i;
 		break;
 	}
 
@@ -389,6 +431,11 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 		xxs = &mod->xxs[sample_num];
 
 		xsh[j].length = hio_read32l(f);		/* Sample length */
+
+		/* Sanity check */
+		if (xsh[j].length > MAX_SAMPLE_SIZE)
+			return -1;
+
 		xsh[j].loop_start = hio_read32l(f);	/* Sample loop start */
 		xsh[j].loop_length = hio_read32l(f);	/* Sample loop length */
 		xsh[j].volume = hio_read8(f);		/* Volume */
@@ -490,6 +537,7 @@ static int xm_load(struct module_data *m, HIO_HANDLE *f, const int start)
     int i, j;
     struct xm_file_header xfh;
     char tracker_name[21];
+    int len;
 
     LOAD_INIT();
 
@@ -508,22 +556,35 @@ static int xm_load(struct module_data *m, HIO_HANDLE *f, const int start)
     xfh.tempo = hio_read16l(f);		/* Default tempo */
     xfh.bpm = hio_read16l(f);		/* Default BPM */
 
+    /* Sanity checks */
+    if (xfh.songlen > 255 || xfh.patterns > 255 || xfh.instruments > 255)
+	return -1;
+
+    if (xfh.restart > 255 || xfh.channels > XMP_MAX_CHANNELS)
+        return -1;
+
+    if (xfh.tempo <= 0 || xfh.tempo >= 32 || xfh.bpm < 32 || xfh.bpm > 255)
+	return -1;
+
+    len = xfh.headersz - 0x14;
+    if (len < 0 || len > 256)
+	return -1;
+
     /* Honor header size -- needed by BoobieSqueezer XMs */
-    hio_read(&xfh.order, xfh.headersz - 0x14, 1, f); /* Pattern order table */
+    hio_read(&xfh.order, len, 1, f); /* Pattern order table */
 
     strncpy(mod->name, (char *)xfh.name, 20);
 
     mod->len = xfh.songlen;
-    mod->rst = xfh.restart;
     mod->chn = xfh.channels;
     mod->pat = xfh.patterns;
-    mod->trk = mod->chn * mod->pat + 1;
     mod->ins = xfh.instruments;
+    mod->rst = xfh.restart;
     mod->spd = xfh.tempo;
     mod->bpm = xfh.bpm;
+    mod->trk = mod->chn * mod->pat + 1;
 
     m->quirk |= xfh.flags & XM_LINEAR_PERIOD_MODE ? QUIRK_LINEAR : 0;
-    m->quirk |= QUIRK_FT2LOOP;
 
     memcpy(mod->xxo, xfh.order, mod->len);
     tracker_name[20] = 0;
@@ -535,25 +596,31 @@ static int xm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	    break;
     }
 
+    /* OpenMPT accurately emulates weird FT2 bugs */
+    if (!strncmp(tracker_name, "FastTracker v2.00", 17) ||
+        !strncmp(tracker_name, "OpenMPT ", 8)) {
+	m->quirk |= QUIRK_FT2BUGS;
+    }
+
 #ifndef LIBXMP_CORE_PLAYER
     if (xfh.headersz == 0x0113) {
 	strcpy(tracker_name, "unknown tracker");
-        m->quirk &= ~QUIRK_FT2LOOP;
+        m->quirk &= ~QUIRK_FT2BUGS;
     } else if (*tracker_name == 0) {
 	strcpy(tracker_name, "Digitrakker");	/* best guess */
-        m->quirk &= ~QUIRK_FT2LOOP;
+        m->quirk &= ~QUIRK_FT2BUGS;
     }
 
     /* See MMD1 loader for explanation */
     if (!strncmp(tracker_name, "MED2XM by J.Pynnone", 19)) {
 	if (mod->bpm <= 10)
 	    mod->bpm = 125 * (0x35 - mod->bpm * 2) / 33;
-        m->quirk &= ~QUIRK_FT2LOOP;
+        m->quirk &= ~QUIRK_FT2BUGS;
     }
 
     if (!strncmp(tracker_name, "FastTracker v 2.00", 18)) {
 	strcpy(tracker_name, "old ModPlug Tracker");
-        m->quirk &= ~QUIRK_FT2LOOP;
+        m->quirk &= ~QUIRK_FT2BUGS;
     }
 
     set_type(m, "%s XM %d.%02d", tracker_name,
