@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -416,13 +416,14 @@ void mixer_softmixer(struct context_data *ctx)
 			if (vi->pos >= vi->end) {
 				samples = 0;
 			} else {
-				samples = 1 + (((int64)(vi->end - vi->pos) <<
+				int64 s = 1 + (((int64)(vi->end - vi->pos) <<
 					SMIX_SHIFT) - vi->frac) / step;
-			}
+				/* ...inside the tick boundaries */
+				if (s > size) {
+					s = size;
+				}
 
-			/* ...inside the tick boundaries */
-			if (samples > size) {
-				samples = size;
+				samples = s;
 			}
 
 			if (vi->vol) {
@@ -453,7 +454,7 @@ void mixer_softmixer(struct context_data *ctx)
 				mix_fn = (*mixers)[mixer];
 
 				/* Call the output handler */
-				if (samples >= 0) {
+				if (samples >= 0 && vi->sptr != NULL) {
 					mix_fn(vi, buf_pos, samples, vol_l,
 								vol_r, step);
 					buf_pos += mix_size;
