@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -59,7 +59,6 @@ static const struct retrig_control rval[] = {
 	
 };
 
-static const struct xmp_event empty_event = { 0, 0, 0, 0, 0, 0, 0 };
 
 /*
  * "Anyway I think this is the most brilliant piece of crap we
@@ -110,6 +109,22 @@ static void update_invloop(struct module_data *m, struct channel_data *xc)
 
 #endif
 
+static int is_first_frame(struct context_data *ctx)
+{
+	struct player_data *p = &ctx->p;
+#ifndef LIBXMP_CORE_DISABLE_IT
+	struct module_data *m = &ctx->m;
+
+	if (m->read_event_type == READ_EVENT_IT) {
+		return p->frame % p->speed == 0;
+	} else {
+		return p->frame == 0;
+	}
+#else
+	return p->frame == 0;
+#endif
+}
+
 static void reset_channels(struct context_data *ctx)
 {
 	struct player_data *p = &ctx->p;
@@ -129,13 +144,19 @@ static void reset_channels(struct context_data *ctx)
 		memset(xc, 0, sizeof (struct channel_data));
 		xc->extra = extra;
 		reset_channel_extras(ctx, xc);
-		xc->ins = xc->key = -1;
+		xc->ins = 0;
+		xc->old_ins = 1;	/* raw value */
+		xc->key = -1;
+		xc->volume = m->volbase;
 	}
 #else
 	for (i = 0; i < p->virt.virt_channels; i++) {
 		xc = &p->xc_data[i];
 		memset(xc, 0, sizeof (struct channel_data));
-		xc->ins = xc->key = -1;
+		xc->ins = 0;
+		xc->old_ins = 1;	/* raw value */
+		xc->key = -1;
+		xc->volume = m->volbase;
 	}
 #endif
 
@@ -185,6 +206,8 @@ static int check_delay(struct context_data *ctx, struct xmp_event *e, int chn)
 	}
 
 	if (HAS_QUIRK(QUIRK_RTDELAY)) {
+		if (e->vol == 0 && e->f2t == 0 && e->ins == 0 && e->note == 0)
+			xc->delayed_event.vol = xc->volume + 1;
 		if (e->note == 0)
 			xc->delayed_event.note = xc->key + 1;
 		if (e->ins == 0)
@@ -201,19 +224,38 @@ static inline void read_row(struct context_data *ctx, int pat, int row)
 	struct xmp_module *mod = &m->mod;
 	struct player_data *p = &ctx->p;
 	struct flow_control *f = &p->flow;
-	struct xmp_event *event;
+	struct xmp_event ev;
 
 	for (chn = 0; chn < mod->chn; chn++) {
 		const int num_rows = mod->xxt[TRACK_NUM(pat, chn)]->rows;
 		if (row < num_rows) {
-			event = &EVENT(pat, chn, row);
+			memcpy(&ev, &EVENT(pat, chn, row), sizeof(ev));
 		} else {
-			event = (struct xmp_event *)&empty_event;
+			memset(&ev, 0, sizeof(ev));
 		}
 
-		if (check_delay(ctx, event, chn) == 0) {
+		if (ev.note == XMP_KEY_OFF) {
+			int env_on = 0;
+			int ins = ev.ins - 1;
+
+			if (IS_VALID_INSTRUMENT(ins) &&
+			    (mod->xxi[ins].aei.flg & XMP_ENVELOPE_ON)) {
+				env_on = 1;
+			}
+
+			if (ev.fxt == FX_EXTENDED && MSN(ev.fxp) == EX_DELAY) {
+				if (ev.ins && (LSN(ev.fxp) || env_on)) {
+					if (LSN(ev.fxp)) {
+						ev.note = 0;
+					}
+					ev.fxp = ev.fxt = 0;
+				}
+			}
+		}
+
+		if (check_delay(ctx, &ev, chn) == 0) {
 			if (!f->rowdelay_set || f->rowdelay > 0) {
-				read_event(ctx, event, chn);
+				read_event(ctx, &ev, chn);
 #ifndef LIBXMP_CORE_PLAYER
 				med_hold_hack(ctx, pat, chn, row);
 #endif
@@ -304,8 +346,12 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 			SET_NOTE(NOTE_END);
 		SET_NOTE(NOTE_ENV_END);
 	}
-	xc->v_idx = update_envelope(&instrument->aei, xc->v_idx, DOENV_RELEASE,
-						HAS_QUIRK(QUIRK_ENVSUS));
+
+	if (!TEST_PER(VENV_PAUSE)) {
+		xc->v_idx = update_envelope(&instrument->aei, xc->v_idx,
+			DOENV_RELEASE, m->read_event_type == READ_EVENT_IT);
+	}
+
 
 	/* If note ended in background channel, we can safely reset it */
 	if (TEST_NOTE(NOTE_END) && chn >= p->virt.num_tracks) {
@@ -320,8 +366,8 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 #endif
 
 	if (TEST(TREMOLO)) {
-		finalvol += get_lfo(&xc->tremolo.lfo, 1 << 6);
-		if (p->frame % p->speed != 0 || HAS_QUIRK(QUIRK_VIBALL)) {
+		finalvol += get_lfo(ctx, &xc->tremolo.lfo, 1 << 6, 0);
+		if (!is_first_frame(ctx) || HAS_QUIRK(QUIRK_VIBALL)) {
 			update_lfo(&xc->tremolo.lfo);
 		}
 	}
@@ -401,8 +447,10 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	instrument = get_instrument(ctx, xc->ins);
 
 	frq_envelope = get_envelope(&instrument->fei, xc->f_idx, 0);
-	xc->f_idx = update_envelope(&instrument->fei, xc->f_idx, DOENV_RELEASE,
-						HAS_QUIRK(QUIRK_ENVSUS));
+	if (!TEST_PER(FENV_PAUSE)) {
+		xc->f_idx = update_envelope(&instrument->fei, xc->f_idx,
+			DOENV_RELEASE, m->read_event_type == READ_EVENT_IT);
+	}
 
 #ifndef LIBXMP_CORE_PLAYER
 	/* Do note slide */
@@ -419,7 +467,7 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 #endif
 
 	/* Instrument vibrato */
-	vibrato = get_lfo(&xc->insvib.lfo, (1024 * (1 + xc->insvib.sweep)));
+	vibrato = get_lfo(ctx, &xc->insvib.lfo, (1024 * (1 + xc->insvib.sweep)), 1);
 	update_lfo(&xc->insvib.lfo);
 	if (xc->insvib.sweep > 1) {
 		xc->insvib.sweep -= 2;
@@ -430,7 +478,7 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	/* Vibrato */
 	if (TEST(VIBRATO) || TEST_PER(VIBRATO)) {
 		int shift = HAS_QUIRK(QUIRK_VIBHALF) ? 10 : 9;
-		int vib = get_lfo(&xc->vibrato.lfo, 1 << shift);
+		int vib = get_lfo(ctx, &xc->vibrato.lfo, 1 << shift, 1);
 
 		if (HAS_QUIRK(QUIRK_VIBINV)) {
 			vibrato -= vib;
@@ -438,7 +486,7 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 			vibrato += vib;
 		}
 
-		if (p->frame % p->speed != 0 || HAS_QUIRK(QUIRK_VIBALL)) {
+		if (!is_first_frame(ctx) || HAS_QUIRK(QUIRK_VIBALL)) {
 			update_lfo(&xc->vibrato.lfo);
 		}
 	}
@@ -532,11 +580,13 @@ static void process_pan(struct context_data *ctx, int chn, int act)
 	instrument = get_instrument(ctx, xc->ins);
 
 	pan_envelope = get_envelope(&instrument->pei, xc->p_idx, 32);
-	xc->p_idx = update_envelope(&instrument->pei, xc->p_idx, DOENV_RELEASE,
-						HAS_QUIRK(QUIRK_ENVSUS));
+	if (!TEST_PER(PENV_PAUSE)) {
+		xc->p_idx = update_envelope(&instrument->pei, xc->p_idx,
+			DOENV_RELEASE, m->read_event_type == READ_EVENT_IT);
+	}
 
 	if (TEST(PANBRELLO)) {
-		panbrello = get_lfo(&xc->panbrello.lfo, 512);
+		panbrello = get_lfo(ctx, &xc->panbrello.lfo, 512, 0);
 		update_lfo(&xc->panbrello.lfo);
 	}
 
@@ -619,7 +669,7 @@ static void update_frequency(struct context_data *ctx, int chn)
 	struct module_data *m = &ctx->m;
 	struct channel_data *xc = &p->xc_data[chn];
 
-	if (p->frame % p->speed != 0 || HAS_QUIRK(QUIRK_PBALL)) {
+	if (!is_first_frame(ctx) || HAS_QUIRK(QUIRK_PBALL)) {
 		if (TEST(PITCHBEND) || TEST_PER(PITCHBEND)) {
 			xc->period += xc->freq.slide;
 		}
@@ -647,7 +697,7 @@ static void update_frequency(struct context_data *ctx, int chn)
 		} 
 	}
 
-	if (p->frame % p->speed == 0) {
+	if (is_first_frame(ctx)) {
 		if (TEST(FINE_BEND)) {
 			xc->period += xc->freq.fslide;
 		}
@@ -655,8 +705,9 @@ static void update_frequency(struct context_data *ctx, int chn)
 #ifndef LIBXMP_CORE_PLAYER
 		if (TEST(FINE_NSLIDE)) {
 			xc->note += xc->noteslide.fslide;
-			xc->period = note_to_period(xc->note, xc->finetune,
-					HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+			xc->period = note_to_period(xc->note,
+				xc->finetune, HAS_QUIRK(QUIRK_LINEAR),
+				xc->per_adj);
 		}
 #endif
 	}
@@ -686,7 +737,7 @@ static void update_pan(struct context_data *ctx, int chn)
 	struct channel_data *xc = &p->xc_data[chn];
 
 	if (TEST(PAN_SLIDE)) {
-		if (p->frame % p->speed == 0) {
+		if (is_first_frame(ctx)) {
 			xc->pan.val += xc->pan.fslide;
 		} else {
 			xc->pan.val += xc->pan.slide;
@@ -712,7 +763,7 @@ static void play_channel(struct context_data *ctx, int chn)
 	xc->info_finalvol = 0;
 
 	/* IT tempo slide */
-	if (TEST(TEMPO_SLIDE) && p->frame) {
+	if (!is_first_frame(ctx) && TEST(TEMPO_SLIDE)) {
 		p->bpm += xc->tempo.slide;
 		CLAMP(p->bpm, 0x20, 0xff);
 	}
@@ -1056,6 +1107,23 @@ int xmp_start_player(xmp_context opaque, int rate, int format)
 	return ret;
 }
 
+static void check_end_of_module(struct context_data *ctx)
+{
+	struct player_data *p = &ctx->p;
+	struct flow_control *f = &p->flow;
+
+	/* check end of module */
+	if (p->ord == p->scan[p->sequence].ord &&
+			p->row == p->scan[p->sequence].row) {
+		if (f->end_point == 0) {
+			p->loop_count++;
+			f->end_point = p->scan[p->sequence].num;
+			/* return -1; */
+		}
+		f->end_point--;
+	}
+}
+
 int xmp_play_frame(xmp_context opaque)
 {
 	struct context_data *ctx = (struct context_data *)opaque;
@@ -1117,8 +1185,11 @@ int xmp_play_frame(xmp_context opaque)
 			 * EE2 + D31 ignores D00 in order 1C line 31. Reported
 			 * by The Welder <welder@majesty.net>, Jan 14 2012
 			 */
-			if (f->delay && f->pbreak)
+			if (HAS_QUIRK(QUIRK_PROTRACK) && f->delay && f->pbreak)
+			{
 				next_row(ctx);
+				check_end_of_module(ctx);
+			}
 			next_row(ctx);
 		}
 	}
@@ -1126,16 +1197,8 @@ int xmp_play_frame(xmp_context opaque)
 	/* check new row */
 
 	if (p->frame == 0) {			/* first frame in row */
-		/* check end of module */
-	    	if (p->ord == p->scan[p->sequence].ord &&
-				p->row == p->scan[p->sequence].row) {
-			if (f->end_point == 0) {
-				p->loop_count++;
-				f->end_point = p->scan[p->sequence].num;
-				/* return -1; */
-			}
-			f->end_point--;
-		}
+
+		check_end_of_module(ctx);
 
 		read_row(ctx, mod->xxo[p->ord], p->row);
 
