@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -60,8 +60,25 @@ static struct xmp_subinstrument *get_subinstrument(struct context_data *ctx,
 	return NULL;
 }
 
-static void reset_envelopes(struct context_data *ctx, struct channel_data *xc,
-				int force_cut)
+static void reset_envelopes(struct context_data *ctx, struct channel_data *xc)
+{
+	struct module_data *m = &ctx->m;
+	struct xmp_module *mod = &m->mod;
+
+	if (!IS_VALID_INSTRUMENT(xc->ins))
+		return;
+
+ 	RESET_NOTE(NOTE_ENV_END);
+
+	xc->v_idx = 0;
+	xc->p_idx = 0;
+	xc->f_idx = 0;
+}
+
+#ifndef LIBXMP_CORE_DISABLE_IT
+
+static void reset_envelopes_carry(struct context_data *ctx,
+				struct channel_data *xc)
 {
 	struct module_data *m = &ctx->m;
 	struct xmp_module *mod = &m->mod;
@@ -72,13 +89,6 @@ static void reset_envelopes(struct context_data *ctx, struct channel_data *xc,
 
  	RESET_NOTE(NOTE_ENV_END);
 
-	if (force_cut) {
-		xc->v_idx = 0;
-		xc->p_idx = 0;
-		xc->f_idx = 0;
-		return;
-	}
-
 	xxi = get_instrument(ctx, xc->ins);
 
 	/* Reset envelope positions */
@@ -98,6 +108,8 @@ static void reset_envelopes(struct context_data *ctx, struct channel_data *xc,
 	}
 }
 
+#endif
+
 static void set_effect_defaults(struct context_data *ctx, int note,
 				struct xmp_subinstrument *sub,
 				struct channel_data *xc, int is_toneporta)
@@ -275,7 +287,7 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 
 	set_effect_defaults(ctx, note, sub, xc, is_toneporta);
 	if (e->ins && sub != NULL) {
-		reset_envelopes(ctx, xc, 0);
+		reset_envelopes(ctx, xc);
 	}
 
 	/* Process new volume */
@@ -330,6 +342,16 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	int k00 = 0;
 	struct xmp_event ev;
 
+	/* From the OpenMPT DelayCombination.xm test case:
+         * "Naturally, Fasttracker 2 ignores notes next to an out-of-range
+	 *  note delay. However, to check whether the delay is out of range,
+	 *  it is simply compared against the current song speed, not taking
+	 *  any pattern delays into account."
+	 */
+	if (p->frame >= p->speed) {
+		return 0;
+	}
+
 	memcpy(&ev, e, sizeof (struct xmp_event));
 
 	xc->flags = 0;
@@ -450,19 +472,24 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 
 		if (key == XMP_KEY_OFF) {
 			int env_on = 0;
+			int vol_set = ev.vol != 0 || ev.fxt == FX_VOLSET;
+			int delay_fx = ev.fxt == FX_EXTENDED && ev.fxp == 0xd0;
 
 			/* OpenMPT NoteOffVolume.xm:
 			 * "If an instrument has no volume envelope, a note-off
 			 *  command should cut the sample completely - unless
 			 *  there is a volume command next it. This applies to
 			 *  both volume commands (volume and effect column)."
+			 *
+			 * ...and unless we have a keyoff+delay without setting
+			 * an instrument. See OffDelay.xm.
 			 */
 			if (IS_VALID_INSTRUMENT(xc->ins) &&
 			    (mod->xxi[xc->ins].aei.flg & XMP_ENVELOPE_ON)) {
 				env_on = 1;
 			}
 			
-			if (env_on || (ev.vol == 0 && ev.fxt != FX_VOLSET)) {
+			if (env_on || (!vol_set && (!ev.ins || !delay_fx))) {
 				SET_NOTE(NOTE_RELEASE);
 				use_ins_vol = 0;
 			} else {
@@ -472,7 +499,10 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 			/* See OpenMPT keyoff+instr.xm, pattern 2 row 0x40 */
 			if (env_on && ev.fxt == FX_EXTENDED &&
 			    (ev.fxp >> 4) == EX_DELAY) {
-				RESET_NOTE(NOTE_RELEASE);
+				/* See OpenMPT OffDelay.xm test case */
+				if ((ev.fxp & 0xf) != 0) {
+					RESET_NOTE(NOTE_RELEASE);
+				}
 			}
 		} else if (key == XMP_KEY_FADE) {
 			/* Handle keyoff + instrument case (NoteOff2.xm) */
@@ -560,7 +590,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 		 * But make sure we have an instrument set, see Letting go
 		 * pos 4 chn 20
 		 */
-		reset_envelopes(ctx, xc, 0);
+		reset_envelopes(ctx, xc);
 	}
 
 	/* Process new volume */
@@ -726,7 +756,7 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 
 	set_effect_defaults(ctx, note, sub, xc, is_toneporta);
 	if (e->ins && sub != NULL) {
-		reset_envelopes(ctx, xc, 0);
+		reset_envelopes(ctx, xc);
 	}
 
 	/* Process new volume */
@@ -852,7 +882,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		if (is_release && is_toneporta && !key) {
 			if (HAS_QUIRK(QUIRK_PRENV) || TEST_NOTE(NOTE_SET)) {
 				is_toneporta = 0;
-				reset_envelopes(ctx, xc, 0);
+				reset_envelopes_carry(ctx, xc);
 			}
 		}
 
@@ -1003,7 +1033,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	/* Do after virtual channel copy */
 	if (is_toneporta) {
 		if (HAS_QUIRK(QUIRK_PRENV) && ev.ins) {
-			reset_envelopes(ctx, xc, 0);
+			reset_envelopes_carry(ctx, xc);
 		}
 	}
 
@@ -1012,13 +1042,13 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	 * channel copies in case of NNA (see test/test.it)
 	 */
 	if (ev.ins && TEST_NOTE(NOTE_ENV_END)) {
-		reset_envelopes(ctx, xc, 1);
+		reset_envelopes(ctx, xc);
 	}
 
 	if (IS_VALID_INSTRUMENT(candidate_ins)) {
 		if (xc->ins != candidate_ins) {
 			/* Reset envelopes if instrument changes */
-			reset_envelopes(ctx, xc, 1);
+			reset_envelopes(ctx, xc);
 		}
 		xc->ins = candidate_ins;
 		xc->ins_fade = mod->xxi[candidate_ins].rls;
@@ -1031,7 +1061,11 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		if (note >= 0) {
 			if (sub->pan >= 0)
 				xc->pan.val = sub->pan;
-			reset_envelopes(ctx, xc, TEST_NOTE(NOTE_CUT));
+			if (TEST_NOTE(NOTE_CUT)) {
+				reset_envelopes(ctx, xc);
+			} else {
+				reset_envelopes_carry(ctx, xc);
+			}
 			RESET_NOTE(NOTE_CUT);
 		} else if (ev.ins) {
 			if (sub->pan >= 0)
@@ -1208,7 +1242,7 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 	}
 
 	if (e->ins && sub != NULL) {
-		reset_envelopes(ctx, xc, 0);
+		reset_envelopes(ctx, xc);
 	}
 
 	/* Process new volume */
@@ -1316,7 +1350,7 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 	set_period(ctx, note, sub, xc, 0);
 
 	if (e->ins && sub != NULL) {
-		reset_envelopes(ctx, xc, 0);
+		reset_envelopes(ctx, xc);
 	}
 
 	xc->volume = e->vol - 1;
